@@ -161,35 +161,6 @@ move_process_to_cgroup (pid_t pid, const char *subsystem, const char *path, libc
   return ret;
 }
 
-int
-libcrun_get_current_unified_cgroup (char **path, bool absolute, libcrun_error_t *err)
-{
-  cleanup_free char *content = NULL;
-  size_t content_size;
-  char *from, *to;
-  int ret;
-
-  ret = read_all_file (PROC_SELF_CGROUP, &content, &content_size, err);
-  if (UNLIKELY (ret < 0))
-    return ret;
-
-  from = strstr (content, "0::");
-  if (UNLIKELY (from == NULL))
-    return crun_make_error (err, 0, "cannot find cgroup2 for the current process");
-
-  from += 3;
-  to = strchr (from, '\n');
-  if (UNLIKELY (to == NULL))
-    return crun_make_error (err, 0, "cannot parse `%s`", PROC_SELF_CGROUP);
-  *to = '\0';
-
-  if (absolute)
-    return append_paths (path, err, CGROUP_ROOT, from, NULL);
-
-  *path = xstrdup (from);
-  return 0;
-}
-
 #ifndef CGROUP2_SUPER_MAGIC
 #  define CGROUP2_SUPER_MAGIC 0x63677270
 #endif
@@ -237,10 +208,61 @@ libcrun_get_cgroup_mode (libcrun_error_t *err)
   return cgroup_mode;
 }
 
+int
+libcrun_get_cgroup_process (pid_t pid, char **path, bool absolute, libcrun_error_t *err)
+{
+  cleanup_free char *content = NULL;
+  char proc_cgroup_file[64];
+  char *cg_path = NULL;
+  size_t content_size;
+  char *controller;
+  char *saveptr;
+  int cgroup_mode;
+  bool has_data;
+  int ret;
+
+  cgroup_mode = libcrun_get_cgroup_mode (err);
+  if (UNLIKELY (cgroup_mode < 0))
+    return cgroup_mode;
+
+  if (pid == 0)
+    strcpy (proc_cgroup_file, PROC_SELF_CGROUP);
+  else
+    sprintf (proc_cgroup_file, "/proc/%d/cgroup", pid);
+
+  ret = read_all_file (proc_cgroup_file, &content, &content_size, err);
+  if (UNLIKELY (ret < 0))
+    return ret;
+
+  for (has_data = read_proc_cgroup (content, &saveptr, NULL, &controller, &cg_path);
+       has_data;
+       has_data = read_proc_cgroup (NULL, &saveptr, NULL, &controller, &cg_path))
+    {
+      if (cgroup_mode == CGROUP_MODE_UNIFIED)
+        {
+          if (strcmp (controller, "") == 0 && strlen (cg_path) > 0)
+            goto found;
+        }
+      else
+        {
+          if (strcmp (controller, "memory"))
+            goto found;
+        }
+    }
+
+  return crun_make_error (err, 0, "cannot find cgroup for the process");
+
+found:
+  if (absolute)
+    return append_paths (path, err, CGROUP_ROOT, cg_path, NULL);
+
+  *path = xstrdup (cg_path);
+  return 0;
+}
+
 static int
 read_pids_cgroup (int dfd, bool recurse, pid_t **pids, size_t *n_pids, size_t *allocated, libcrun_error_t *err)
 {
-  __attribute__ ((unused)) cleanup_close int clean_dfd = dfd;
   cleanup_close int tasksfd = -1;
   cleanup_free char *buffer = NULL;
   char *saveptr = NULL;
@@ -287,11 +309,11 @@ read_pids_cgroup (int dfd, bool recurse, pid_t **pids, size_t *n_pids, size_t *a
       if (UNLIKELY (dir == NULL))
         return crun_make_error (err, errno, "open cgroup sub-directory");
       /* Now dir owns the dfd descriptor.  */
-      clean_dfd = -1;
+      dfd = -1;
 
       for (de = readdir (dir); de; de = readdir (dir))
         {
-          int nfd;
+          cleanup_close int nfd = -1;
 
           if (strcmp (de->d_name, ".") == 0 || strcmp (de->d_name, "..") == 0)
             continue;
@@ -346,22 +368,16 @@ rmdir_all_fd (int dfd)
           size_t i, n_pids = 0, allocated = 0;
           cleanup_close int child_dfd = -1;
           int tmp;
-          int child_dfd_clone;
 
           child_dfd = openat (dfd, name, O_DIRECTORY | O_CLOEXEC);
           if (child_dfd < 0)
             return child_dfd;
 
-          /* read_pids_cgroup takes ownership for the fd, so dup it.  */
-          child_dfd_clone = dup (child_dfd);
-          if (LIKELY (child_dfd_clone >= 0))
+          ret = read_pids_cgroup (child_dfd, true, &pids, &n_pids, &allocated, &tmp_err);
+          if (UNLIKELY (ret < 0))
             {
-              ret = read_pids_cgroup (child_dfd_clone, true, &pids, &n_pids, &allocated, &tmp_err);
-              if (UNLIKELY (ret < 0))
-                {
-                  crun_error_release (&tmp_err);
-                  continue;
-                }
+              crun_error_release (&tmp_err);
+              continue;
             }
 
           for (i = 0; i < n_pids; i++)
@@ -394,8 +410,8 @@ int
 libcrun_cgroup_read_pids_from_path (const char *path, bool recurse, pid_t **pids, libcrun_error_t *err)
 {
   cleanup_free char *cgroup_path = NULL;
+  cleanup_close int dirfd = -1;
   size_t n_pids, allocated;
-  int dirfd;
   int mode;
   int ret;
 
@@ -928,7 +944,7 @@ enable_controllers (const char *path, libcrun_error_t *err)
 }
 
 int
-libcrun_move_process_to_cgroup (pid_t pid, pid_t init_pid, char *path, libcrun_error_t *err)
+libcrun_move_process_to_cgroup (pid_t pid, pid_t init_pid, const char *path, bool create_if_missing, libcrun_error_t *err)
 {
   int cgroup_mode = libcrun_get_cgroup_mode (err);
   if (UNLIKELY (cgroup_mode < 0))
@@ -937,7 +953,7 @@ libcrun_move_process_to_cgroup (pid_t pid, pid_t init_pid, char *path, libcrun_e
   if (path == NULL || *path == '\0')
     return 0;
 
-  return enter_cgroup (cgroup_mode, pid, init_pid, path, false, err);
+  return enter_cgroup (cgroup_mode, pid, init_pid, path, create_if_missing, err);
 }
 
 int
@@ -971,3 +987,76 @@ libcrun_get_cgroup_dirfd (struct libcrun_cgroup_status *status, const char *sub_
 
   return cgroupdirfd;
 }
+
+int
+libcrun_migrate_all_pids_to_cgroup (pid_t init_pid, char *from, char *to, libcrun_error_t *err)
+{
+  cleanup_free pid_t *pids = NULL;
+  cleanup_close int child_dfd = -1;
+  int cgroup_mode;
+  size_t from_len;
+  size_t i;
+  int ret;
+
+  cgroup_mode = libcrun_get_cgroup_mode (err);
+  if (cgroup_mode < 0)
+    return cgroup_mode;
+
+  ret = libcrun_cgroup_pause_unpause_path (from, true, err);
+  if (UNLIKELY (ret < 0))
+    return ret;
+
+  ret = libcrun_cgroup_read_pids_from_path (from, true, &pids, err);
+  if (UNLIKELY (ret < 0))
+    return ret;
+
+  from_len = strlen (from);
+
+  for (i = 0; pids && pids[i]; i++)
+    {
+      cleanup_free char *pid_path = NULL;
+      cleanup_free char *dest_cgroup = NULL;
+
+      ret = libcrun_get_cgroup_process (pids[i], &pid_path, false, err);
+      if (UNLIKELY (ret < 0))
+        return ret;
+
+      /* Make sure the pid is in the cgroup we are migrating from.  */
+      if (! has_prefix (pid_path, from))
+        return crun_make_error (err, 0, "error migrating pid %d.  It is not in the cgroup `%s`", pids[i], from);
+
+      /* Build the destination cgroup path, keeping the same hierarchy.  */
+      xasprintf (&dest_cgroup, "%s%s", to, pid_path + from_len);
+
+      ret = enter_cgroup (cgroup_mode, pids[i], init_pid, dest_cgroup, false, err);
+      if (UNLIKELY (ret < 0))
+        return ret;
+    }
+
+  ret = libcrun_cgroup_pause_unpause_path (from, false, err);
+  if (UNLIKELY (ret < 0))
+    return ret;
+
+  return destroy_cgroup_path (from, cgroup_mode, err);
+}
+
+int
+get_cgroup_dirfd_path (int dirfd, char **path, libcrun_error_t *err)
+{
+  cleanup_free char *cgroup_path = NULL;
+  proc_fd_path_t fd_path;
+  ssize_t len;
+
+  get_proc_self_fd_path (fd_path, dirfd);
+
+  len = safe_readlinkat (AT_FDCWD, fd_path, &cgroup_path, 0, err);
+  if (UNLIKELY (len < 0))
+    return len;
+
+  if (has_prefix (cgroup_path, CGROUP_ROOT))
+    {
+      *path = xstrdup (cgroup_path + strlen (CGROUP_ROOT));
+      return 0;
+    }
+  return crun_make_error (err, 0, "invalid cgroup path `%s`", cgroup_path);
+}
