@@ -36,7 +36,9 @@
 #  include "cgroup.h"
 #  include "cgroup-utils.h"
 
-#  include <dlfcn.h>
+#  ifndef STATIC
+#    include <dlfcn.h>
+#  endif
 
 #  define CRIU_CHECKPOINT_LOG_FILE "dump.log"
 #  define CRIU_RESTORE_LOG_FILE "restore.log"
@@ -86,6 +88,7 @@ struct libcriu_wrapper_s
   int (*criu_set_parent_images) (const char *path);
   void (*criu_set_pid) (int pid);
   int (*criu_set_root) (const char *root);
+  int (*criu_add_cg_root) (const char *ctrl, const char *path);
   void (*criu_set_shell_job) (bool shell_job);
   void (*criu_set_tcp_established) (bool tcp_established);
   void (*criu_set_track_mem) (bool track_mem);
@@ -105,8 +108,10 @@ cleanup_wrapper (void *p)
   if (*w == NULL)
     return;
 
+#  ifndef STATIC
   if ((*w)->handle)
     dlclose ((*w)->handle);
+#  endif
   free (*w);
   libcriu_wrapper = NULL;
 }
@@ -118,20 +123,27 @@ load_wrapper (struct libcriu_wrapper_s **wrapper_out, libcrun_error_t *err)
 {
   cleanup_free struct libcriu_wrapper_s *wrapper = xmalloc0 (sizeof (*wrapper));
 
-#  define LOAD_CRIU_FUNCTION(X, ALLOW_NULL)                                                    \
-    do                                                                                         \
-      {                                                                                        \
-        wrapper->X = dlsym (wrapper->handle, #X);                                              \
-        if (! ALLOW_NULL && wrapper->X == NULL)                                                \
-          {                                                                                    \
-            dlclose (wrapper->handle);                                                         \
-            return crun_make_error (err, 0, "could not find symbol `%s` in `libcriu.so`", #X); \
-          }                                                                                    \
-    } while (0)
+#  ifdef STATIC
+#    define LOAD_CRIU_FUNCTION(X, ALLOW_NULL) \
+      wrapper->X = &X;
+#  else
+#    define LOAD_CRIU_FUNCTION(X, ALLOW_NULL)                                                    \
+      do                                                                                         \
+        {                                                                                        \
+          wrapper->X = dlsym (wrapper->handle, #X);                                              \
+          if (! ALLOW_NULL && wrapper->X == NULL)                                                \
+            {                                                                                    \
+              dlclose (wrapper->handle);                                                         \
+              return crun_make_error (err, 0, "could not find symbol `%s` in `libcriu.so`", #X); \
+            }                                                                                    \
+      } while (0)
+#  endif
 
+#  ifndef STATIC
   wrapper->handle = dlopen ("libcriu.so.2", RTLD_NOW);
   if (wrapper->handle == NULL)
     return crun_make_error (err, 0, "could not load `libcriu.so.2`");
+#  endif
 
   LOAD_CRIU_FUNCTION (criu_add_ext_mount, false);
   LOAD_CRIU_FUNCTION (criu_add_external, false);
@@ -170,6 +182,7 @@ load_wrapper (struct libcriu_wrapper_s **wrapper_out, libcrun_error_t *err)
   LOAD_CRIU_FUNCTION (criu_set_parent_images, false);
   LOAD_CRIU_FUNCTION (criu_set_pid, false);
   LOAD_CRIU_FUNCTION (criu_set_root, false);
+  LOAD_CRIU_FUNCTION (criu_add_cg_root, false);
   LOAD_CRIU_FUNCTION (criu_set_shell_job, false);
   LOAD_CRIU_FUNCTION (criu_set_tcp_established, false);
   LOAD_CRIU_FUNCTION (criu_set_track_mem, false);
@@ -1020,6 +1033,19 @@ libcrun_container_restore_linux_criu (libcrun_container_status_t *status, libcru
   libcriu_wrapper->criu_set_tcp_established (cr_options->tcp_established);
   libcriu_wrapper->criu_set_file_locks (cr_options->file_locks);
   libcriu_wrapper->criu_set_orphan_pts_master (true);
+
+  if (status->cgroup_path)
+    {
+      ret = libcriu_wrapper->criu_add_cg_root (NULL, status->cgroup_path);
+      if (UNLIKELY (ret != 0))
+        return crun_make_error (err, 0, "error setting CRIU cgroup root to `%s`", status->cgroup_path);
+    }
+
+  if (cr_options->manage_cgroups_mode == -1)
+    /* Defaulting to CRIU_CG_MODE_SOFT just as runc */
+    libcriu_wrapper->criu_set_manage_cgroups_mode (CRIU_CG_MODE_SOFT);
+  else
+    libcriu_wrapper->criu_set_manage_cgroups_mode (cr_options->manage_cgroups_mode);
   libcriu_wrapper->criu_set_manage_cgroups (true);
 
   if (libcriu_wrapper->criu_set_network_lock && cr_options->network_lock_method > 0)
