@@ -71,7 +71,10 @@ crun_error_wrap (libcrun_error_t *err, const char *fmt, ...)
   int ret;
 
   if (err == NULL || *err == NULL)
-    return 0;
+    {
+      // Internal error
+      return 0;
+    }
 
   ret = -(*err)->status - 1;
 
@@ -123,8 +126,11 @@ crun_error_write_warning_and_release (FILE *out, libcrun_error_t **err)
 
   if (out == NULL)
     out = stderr;
-  if (err == NULL || *err == NULL)
-    return;
+  if (err == NULL || *err == NULL || **err == NULL)
+    {
+      // Internal error
+      return;
+    }
 
   ref = **err;
   if (ref->status)
@@ -291,7 +297,7 @@ log_write_to_stderr (int errno_, const char *msg, int verbosity, void *arg arg_u
 void
 log_write_to_syslog (int errno_, const char *msg, int verbosity, void *arg arg_unused)
 {
-  int priority;
+  int priority = LOG_ERR;
   switch (verbosity)
     {
     case LIBCRUN_VERBOSITY_DEBUG:
@@ -317,7 +323,7 @@ log_write_to_journald (int errno_, const char *msg, int verbosity, void *arg arg
   (void) msg;
   (void) verbosity;
 #ifdef HAVE_SYSTEMD
-  int priority;
+  int priority = LOG_ERR;
   switch (verbosity)
     {
     case LIBCRUN_VERBOSITY_DEBUG:
