@@ -50,35 +50,52 @@ struct krun_config
   void *handle;
   void *handle_sev;
   bool sev;
+  int32_t ctx_id;
+  int32_t ctx_id_sev;
 };
 
 /* libkrun handler.  */
 #if HAVE_DLOPEN && HAVE_LIBKRUN
+static int32_t
+libkrun_create_context (void *handle, libcrun_error_t *err)
+{
+  int32_t (*krun_create_ctx) ();
+  int32_t ctx_id;
+
+  krun_create_ctx = dlsym (handle, "krun_create_ctx");
+  if (krun_create_ctx == NULL)
+    return crun_make_error (err, 0, "could not find symbol in the krun library");
+
+  ctx_id = krun_create_ctx ();
+  if (UNLIKELY (ctx_id < 0))
+    return crun_make_error (err, -ctx_id, "could not create krun context");
+
+  return ctx_id;
+}
+
 static int
 libkrun_exec (void *cookie, libcrun_container_t *container, const char *pathname, char *const argv[])
 {
   runtime_spec_schema_config_schema *def = container->container_def;
   int32_t (*krun_set_log_level) (uint32_t level);
-  int32_t (*krun_create_ctx) ();
   int (*krun_start_enter) (uint32_t ctx_id);
   int32_t (*krun_set_vm_config) (uint32_t ctx_id, uint8_t num_vcpus, uint32_t ram_mib);
   int32_t (*krun_set_root) (uint32_t ctx_id, const char *root_path);
   int32_t (*krun_set_root_disk) (uint32_t ctx_id, const char *disk_path);
   int32_t (*krun_set_workdir) (uint32_t ctx_id, const char *workdir_path);
-  int32_t (*krun_set_exec) (uint32_t ctx_id, const char *exec_path, char *const argv[], char *const envp[]);
   int32_t (*krun_set_tee_config_file) (uint32_t ctx_id, const char *file_path);
   struct krun_config *kconf = (struct krun_config *) cookie;
   void *handle;
   uint32_t num_vcpus, ram_mib;
   int32_t ctx_id, ret;
   cpu_set_t set;
-  char *const envp[] = { 0 };
 
   if (access ("/krun-sev.json", F_OK) == 0)
     {
       if (kconf->handle_sev == NULL)
         error (EXIT_FAILURE, 0, "the container requires libkrun-sev but it's not available");
       handle = kconf->handle_sev;
+      ctx_id = kconf->ctx_id_sev;
       kconf->sev = true;
     }
   else
@@ -86,23 +103,18 @@ libkrun_exec (void *cookie, libcrun_container_t *container, const char *pathname
       if (kconf->handle == NULL)
         error (EXIT_FAILURE, 0, "the container requires libkrun but it's not available");
       handle = kconf->handle;
+      ctx_id = kconf->ctx_id;
       kconf->sev = false;
     }
 
   krun_set_log_level = dlsym (handle, "krun_set_log_level");
-  krun_create_ctx = dlsym (handle, "krun_create_ctx");
   krun_start_enter = dlsym (handle, "krun_start_enter");
-  if (krun_set_log_level == NULL || krun_create_ctx == NULL
-      || krun_start_enter == NULL)
-    error (EXIT_FAILURE, 0, "could not find symbol in `libkrun.so`");
+  if (krun_set_log_level == NULL || krun_start_enter == NULL)
+    error (EXIT_FAILURE, 0, "could not find symbol in the krun library");
 
   /* Set log level to "error" */
   krun_set_log_level (1);
 
-  ctx_id = krun_create_ctx ();
-  if (UNLIKELY (ctx_id < 0))
-    error (EXIT_FAILURE, -ctx_id, "could not create krun context");
-
   if (kconf->sev)
     {
       krun_set_root_disk = dlsym (handle, "krun_set_root_disk");
@@ -136,9 +148,7 @@ libkrun_exec (void *cookie, libcrun_container_t *container, const char *pathname
       krun_set_vm_config = dlsym (handle, "krun_set_vm_config");
       krun_set_root = dlsym (handle, "krun_set_root");
       krun_set_workdir = dlsym (handle, "krun_set_workdir");
-      krun_set_exec = dlsym (handle, "krun_set_exec");
-      if (krun_set_vm_config == NULL || krun_set_root == NULL
-          || krun_set_exec == NULL)
+      if (krun_set_vm_config == NULL || krun_set_root == NULL)
         error (EXIT_FAILURE, 0, "could not find symbol in `libkrun.so`");
 
       ret = krun_set_vm_config (ctx_id, num_vcpus, ram_mib);
@@ -155,13 +165,9 @@ libkrun_exec (void *cookie, libcrun_container_t *container, const char *pathname
           if (UNLIKELY (ret < 0))
             error (EXIT_FAILURE, -ret, "could not set krun working directory");
         }
-
-      ret = krun_set_exec (ctx_id, pathname, &argv[1], &envp[0]);
-      if (UNLIKELY (ret < 0))
-        error (EXIT_FAILURE, -ret, "could not set krun executable");
     }
-
-  return krun_start_enter (ctx_id);
+  ret = krun_start_enter (ctx_id);
+  return -ret;
 }
 
 /* libkrun_create_kvm_device: explicitly adds kvm device.  */
@@ -212,7 +218,7 @@ libkrun_configure_container (void *cookie, enum handler_configure_phase phase,
 
       /* CVE-2025-24965: the content below rootfs cannot be trusted because it is controlled by the user.  We
          must ensure the file is opened below the rootfs directory.  */
-      fd = safe_openat (rootfsfd, rootfs, KRUN_CONFIG_FILE, WRITE_FILE_DEFAULT_FLAGS | O_NOFOLLOW, 0700, err);
+      fd = safe_openat (rootfsfd, rootfs, KRUN_CONFIG_FILE, WRITE_FILE_DEFAULT_FLAGS | O_NOFOLLOW, S_IRUSR | S_IRGRP | S_IROTH, err);
       if (UNLIKELY (fd < 0))
         return fd;
 
@@ -267,6 +273,7 @@ libkrun_configure_container (void *cookie, enum handler_configure_phase phase,
 static int
 libkrun_load (void **cookie, libcrun_error_t *err)
 {
+  int32_t ret;
   struct krun_config *kconf;
   const char *libkrun_so = "libkrun.so.1";
   const char *libkrun_sev_so = "libkrun-sev.so.1";
@@ -286,6 +293,32 @@ libkrun_load (void **cookie, libcrun_error_t *err)
 
   kconf->sev = false;
 
+  /* Newer versions of libkrun no longer link against libkrunfw and
+     instead they open it when creating the context. This implies
+     we need to call "krun_create_ctx" before switching namespaces
+     or it won't be able to find the library bundling the kernel. */
+  if (kconf->handle)
+    {
+      ret = libkrun_create_context (kconf->handle, err);
+      if (UNLIKELY (ret < 0))
+        {
+          free (kconf);
+          return ret;
+        }
+      kconf->ctx_id = ret;
+    }
+
+  if (kconf->handle_sev)
+    {
+      ret = libkrun_create_context (kconf->handle_sev, err);
+      if (UNLIKELY (ret < 0))
+        {
+          free (kconf);
+          return ret;
+        }
+      kconf->ctx_id_sev = ret;
+    }
+
   *cookie = kconf;
 
   return 0;
@@ -296,11 +329,21 @@ libkrun_unload (void *cookie, libcrun_error_t *err)
 {
   int r;
 
-  if (cookie)
+  struct krun_config *kconf = (struct krun_config *) cookie;
+  if (kconf != NULL)
     {
-      r = dlclose (cookie);
-      if (UNLIKELY (r < 0))
-        return crun_make_error (err, 0, "could not unload handle: `%s`", dlerror ());
+      if (kconf->handle != NULL)
+        {
+          r = dlclose (kconf->handle);
+          if (UNLIKELY (r != 0))
+            return crun_make_error (err, 0, "could not unload handle: `%s`", dlerror ());
+        }
+      if (kconf->handle_sev != NULL)
+        {
+          r = dlclose (kconf->handle_sev);
+          if (UNLIKELY (r != 0))
+            return crun_make_error (err, 0, "could not unload handle_sev: `%s`", dlerror ());
+        }
     }
   return 0;
 }
