@@ -584,6 +584,7 @@ crun_safe_create_and_open_ref_at (bool dir, int dirfd, const char *dirpath, cons
   if (LIKELY (fd >= 0))
     return fd;
 
+  crun_error_release (err);
   return crun_safe_ensure_at (true, dir, dirfd, dirpath, path, mode, MAX_READLINKS, err);
 }
 
@@ -1462,7 +1463,7 @@ set_home_env (uid_t id)
 
 error:
   /* Let callers handle the error if the user was not found. */
-  return ret;
+  return ret ? -errno : 0;
 }
 
 /*if subuid or subgid exist, take the first range for the user */
@@ -1848,50 +1849,70 @@ parse_json_file (yajl_val *out, const char *jsondata, struct parser_context *ctx
   return 0;
 }
 
+#define CHECK_ACCESS_NOT_EXECUTABLE 1
+#define CHECK_ACCESS_NOT_REGULAR 2
+
+/* check that the specified path exists and it is executable.
+   Return
+   - 0 if the file is executable
+   - CHECK_ACCESS_NOT_EXECUTABLE if it exists but it is not executable
+   - CHECK_ACCESS_NOT_REGULAR if it not a regular file
+   - -errno for any other generic error
+ */
 static int
 check_access (const char *path)
 {
   int ret;
   mode_t mode;
 
-#ifdef ANDROID
-  ret = access (path, X_OK);
+#ifdef HAVE_EACCESS
+#  define ACCESS(path, mode) (eaccess (path, mode))
 #else
-  ret = eaccess (path, X_OK);
+#  define ACCESS(path, mode) (access (path, mode))
 #endif
+
+  ret = ACCESS (path, X_OK);
   if (ret < 0)
-    return ret;
+    {
+      /* If the file is not executable, check if it exists.  */
+      if (errno == EACCES)
+        {
+          int saved_errno = errno;
+          ret = ACCESS (path, F_OK);
+          errno = saved_errno;
+
+          if (ret == 0)
+            return CHECK_ACCESS_NOT_EXECUTABLE;
+        }
+      return -errno;
+    }
 
   ret = get_file_type (&mode, false, path);
   if (UNLIKELY (ret < 0))
-    return ret;
+    return -errno;
 
   if (! S_ISREG (mode))
-    {
-      errno = EPERM;
-      return -1;
-    }
+    return CHECK_ACCESS_NOT_REGULAR;
 
+  /* It exists, is executable and is a regular file.  */
   return 0;
 }
 
-char *
-find_executable (const char *executable_path, const char *cwd)
+int
+find_executable (char **out, const char *executable_path, const char *cwd, libcrun_error_t *err)
 {
   cleanup_free char *cwd_executable_path = NULL;
   cleanup_free char *tmp = NULL;
-  char path[PATH_MAX + 1];
-  int last_error = ENOENT;
+  int last_error = -ENOENT;
   char *it, *end;
   int ret;
 
-  if (executable_path == NULL)
-    {
-      errno = EINVAL;
-      return NULL;
-    }
+  *out = NULL;
 
-  if (executable_path[0] == '.' || (executable_path[0] != '/' && strchr (executable_path, '/')))
+  if (executable_path == NULL || executable_path[0] == '\0')
+    return crun_make_error (err, ENOENT, "cannot find `` in $PATH");
+
+  if (executable_path[0] != '/' && strchr (executable_path, '/'))
     {
       cleanup_free char *cwd_allocated = NULL;
 
@@ -1906,7 +1927,9 @@ find_executable (const char *executable_path, const char *cwd)
 
       /* Make sure the path starts with a '/' so it will hit the check
          for absolute paths.  */
-      xasprintf (&cwd_executable_path, "%s%s/%s", cwd[0] == '/' ? "" : "/", cwd, executable_path);
+      ret = append_paths (&cwd_executable_path, err, "/", cwd, executable_path, NULL);
+      if (UNLIKELY (ret < 0))
+        return ret;
       executable_path = cwd_executable_path;
     }
 
@@ -1914,36 +1937,61 @@ find_executable (const char *executable_path, const char *cwd)
   if (executable_path[0] == '/')
     {
       ret = check_access (executable_path);
-      if (ret == 0)
-        return xstrdup (executable_path);
-      return NULL;
+      if (LIKELY (ret == 0))
+        {
+          *out = xstrdup (executable_path);
+          return 0;
+        }
+      last_error = ret;
+      goto fail;
     }
 
   end = tmp = xstrdup (getenv ("PATH"));
 
   while ((it = strsep (&end, ":")))
     {
-      size_t len;
+      cleanup_free char *path = NULL;
 
       if (it == end)
         it = ".";
 
-      len = snprintf (path, PATH_MAX, "%s/%s", it, executable_path);
-      if (len >= PATH_MAX)
-        continue;
+      ret = append_paths (&path, err, it, executable_path, NULL);
+      if (UNLIKELY (ret < 0))
+        {
+          crun_error_release (err);
+          continue;
+        }
 
       ret = check_access (path);
       if (ret == 0)
-        return xstrdup (path);
+        {
+          /* Change owner.  */
+          *out = path;
+          path = NULL;
+          return 0;
+        }
 
-      if (errno == ENOENT)
+      if (ret == -ENOENT)
         continue;
 
-      last_error = errno;
+      last_error = ret;
     }
 
-  errno = last_error;
-  return NULL;
+fail:
+  switch (last_error)
+    {
+    case CHECK_ACCESS_NOT_EXECUTABLE:
+      return crun_make_error (err, EPERM, "the path `%s` exists but it is not executable", executable_path);
+
+    case CHECK_ACCESS_NOT_REGULAR:
+      return crun_make_error (err, EPERM, "the path `%s` is not a regular file", executable_path);
+
+    default:
+      errno = -last_error;
+      if (errno == ENOENT)
+        return crun_make_error (err, errno, "executable file `%s` not found%s", executable_path, executable_path[0] == '/' ? "" : " in $PATH");
+      return crun_make_error (err, errno, "cannot open `%s`", executable_path);
+    }
 }
 
 #ifdef HAVE_FGETXATTR
@@ -2110,11 +2158,11 @@ copy_recursive_fd_to_fd (int srcdirfd, int dfd, const char *srcname, const char
       switch (mode & S_IFMT)
         {
         case S_IFREG:
-          srcfd = openat (dirfd (dsrcfd), de->d_name, O_NONBLOCK | O_RDONLY | O_CLOEXEC);
+          srcfd = openat (dirfd (dsrcfd), de->d_name, O_NONBLOCK | O_RDONLY | O_CLOEXEC | O_NOFOLLOW);
           if (UNLIKELY (srcfd < 0))
             return crun_make_error (err, errno, "open `%s/%s`", srcname, de->d_name);
 
-          destfd = openat (destdirfd, de->d_name, O_RDWR | O_CREAT | O_CLOEXEC, 0777);
+          destfd = openat (destdirfd, de->d_name, O_RDWR | O_CREAT | O_CLOEXEC | O_NOFOLLOW, 0777);
           if (UNLIKELY (destfd < 0))
             return crun_make_error (err, errno, "open `%s/%s`", destname, de->d_name);
 
@@ -2137,11 +2185,11 @@ copy_recursive_fd_to_fd (int srcdirfd, int dfd, const char *srcname, const char
           if (UNLIKELY (ret < 0))
             return crun_make_error (err, errno, "mkdir `%s/%s`", destname, de->d_name);
 
-          srcfd = openat (dirfd (dsrcfd), de->d_name, O_DIRECTORY | O_CLOEXEC);
+          srcfd = openat (dirfd (dsrcfd), de->d_name, O_DIRECTORY | O_CLOEXEC | O_NOFOLLOW);
           if (UNLIKELY (srcfd < 0))
             return crun_make_error (err, errno, "open directory `%s/%s`", srcname, de->d_name);
 
-          destfd = openat (destdirfd, de->d_name, O_DIRECTORY | O_CLOEXEC);
+          destfd = openat (destdirfd, de->d_name, O_DIRECTORY | O_CLOEXEC | O_NOFOLLOW);
           if (UNLIKELY (destfd < 0))
             return crun_make_error (err, errno, "open directory `%s/%s`", srcname, de->d_name);
 
@@ -2202,29 +2250,13 @@ copy_recursive_fd_to_fd (int srcdirfd, int dfd, const char *srcname, const char
   return 0;
 }
 
-const char *
-find_annotation_map (json_map_string_string *annotations, const char *name)
-{
-  size_t i;
-
-  if (annotations == NULL)
-    return NULL;
-
-  for (i = 0; i < annotations->len; i++)
-    {
-      if (strcmp (annotations->keys[i], name) == 0)
-        return annotations->values[i];
-    }
-  return NULL;
-}
-
 const char *
 find_annotation (libcrun_container_t *container, const char *name)
 {
   if (container->container_def->annotations == NULL)
     return NULL;
 
-  return find_annotation_map (container->container_def->annotations, name);
+  return find_string_map_value (container->annotations, name);
 }
 
 int
